@@ -152,6 +152,15 @@ describe('mpack', function()
     end)
   end)
 
+  describe('list-like table with string keys', function()
+    it('ok', function()
+      local unpack = mpack.Unpacker()
+      local pack = mpack.Packer()
+      local tbl = {['1'] = 0, [2] = 1, ['3'] = 3}
+      assert.are_same(tbl, unpack(pack(tbl)))
+    end)
+  end)
+
   describe('very large array', function()
     it('ok', function()
       local arr = {}
@@ -470,4 +479,16 @@ describe('mpack', function()
       assert.are_same("mpack.NIL", tostring(mpack.NIL))
     end)
   end)
+
+  it('should allow to be required twice', function()
+    package.loaded['mpack'] = nil
+    local ok, new_mpack_or_err = pcall(require, 'mpack')
+    assert.is_true(ok, new_mpack_or_err)
+    assert.are.equals(new_mpack_or_err.NIL, mpack.NIL)
+    assert.are.equals(
+      getmetatable(mpack.NIL),
+      getmetatable(new_mpack_or_err.NIL)
+    )
+  end)
+
 end)
